/***************************************************************************
                          qtplgviewerdoc.cpp  -  description
                             -------------------
    begin                : Don Jun 28 10:55:59 CEST 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qmsgbox.h>
#include <qfiledialog.h>


// application specific includes
#include "qtplgviewerdoc.h"
#include "qtplgviewer.h"
#include "qtplgviewerview.h"
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

QtpLgviewerDoc::QtpLgviewerDoc()
{
  pViewList = new QList<QtpLgviewerView>;
  pViewList->setAutoDelete(false);
}

QtpLgviewerDoc::~QtpLgviewerDoc()
{
  delete pViewList;
}

void QtpLgviewerDoc::addView(QtpLgviewerView *view)
{
  pViewList->append(view);
	changedViewList();
}

void QtpLgviewerDoc::removeView(QtpLgviewerView *view)
{
	  pViewList->remove(view);
	  if(!pViewList->isEmpty())
			changedViewList();
		else
			deleteContents();
}

void QtpLgviewerDoc::changedViewList(){	
	
	QtpLgviewerView *w;
	if((int)pViewList->count() == 1){
  	w=pViewList->first();
  	w->slotUpdateName(m_title);
	}
	else{
		int i;
    for( i=1,w=pViewList->first(); w!=0; i++, w=pViewList->next())
  		w->slotUpdateName(QString(m_title+":%1").arg(i));	
	}
}

bool QtpLgviewerDoc::isLastView() {
  return ((int) pViewList->count() == 1);
}


void QtpLgviewerDoc::updateAllViews(QtpLgviewerView *sender)
{
  QtpLgviewerView *w;
  for(w=pViewList->first(); w!=0; w=pViewList->next())
  {
     w->update(sender);
  }

}

void QtpLgviewerDoc::setPathName(const QString &name)
{
  m_filename=name;
	m_title=QFileInfo(name).fileName();
}

const QString& QtpLgviewerDoc::pathName() const
{
  return m_filename;
}

void QtpLgviewerDoc::setTitle(const QString &title)
{
  m_title=title;
}

const QString &QtpLgviewerDoc::title() const
{
  return m_title;
}


void QtpLgviewerDoc::closeDocument()
{
  QtpLgviewerView *w;
  if(!isLastView())
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
   	 	if(!w->close())
 				break;
    }
	}
  if(isLastView())
  {
  	w=pViewList->first();
  	w->close();
  }
}

bool QtpLgviewerDoc::newDocument()
{
	// lock data access mutex
  while(!accessmutex.lock());
 	pointlist.clear();
 	polylist.clear();
 	colortable.clear();
  accessmutex.unlock();

  modified=false;
  return true;
}

bool QtpLgviewerDoc::openDocument(const QString &filename, const char *format /*=0*/)
{

	ifstream f( (const char *) filename );
	if ( !f ) throw QString("Unable to open file!");
	
	unsigned int colors,points,polygons;
	f >> points >> polygons >> colors;
	
	if(!f || f.eof()) throw QString("unexpected end of file!");
	
	// lock access to data
	while(!accessmutex.lock());
	
	try
	{
		pointlist.resize(points);
		polylist.resize(polygons);
		colortable.resize(colors);
	
		if(pointlist.size() != points || polylist.size() != polygons || colortable.size() != colors)
		{
			QString errormsg;
			errormsg.sprintf("Not enough memory for %i point, %i polygons and %i colors!",points,polygons,colors);
			throw errormsg;
		}
		
		if(f.eof()) throw QString("Unexpected end of file!");
		
		// now read point list
		for(vector<MyPoly3D::point4f>::iterator i = pointlist.begin(); i != pointlist.end(); i++)
		{
			f >> i->x() >> i->y() >> i->z();
			i->w() = 1.0;
		}
		
		if(f.eof()) throw QString("Unexpected end of file!");
		
		// ...read polygon list
		for(vector<MyPoly3D::poly>::iterator i = polylist.begin(); i != polylist.end(); i++)
		{
			f >> i->p1 >> i->p2 >> i->p3 >> i->cindex;
			// warning! we convert indicies to c array convertion (first elem got index 0!)
			if(!(--(i->p1) < points && --(i->p2) < points && --(i->p3) < points))
			{
				int errindex = (i->p1 < points) ? i->p2 : i->p1;
				if(errindex < points) errindex = i->p3;
			
				QString errormsg;
				errormsg.sprintf("A Polygon refers to a point %i, but there are only %i points given!",errindex+1,points);
				throw errormsg;
			}
			if(--(i->cindex) >= colors)
			{
				QString errormsg;
				errormsg.sprintf("A Polygon refers to a color %i, but there are only %i colors given!",i->cindex+1,colors);
				throw errormsg;
			}
		}
	
		if(f.eof()) throw QString("Unexpected end of file!");
	
		// ...last but not least, read the color table
		for(vector<MyPoly3D::color3f>::iterator i = colortable.begin(); i != colortable.end(); i++)
		{
			f >> i->r >> i->g >> i->b;
			if(i->r > 1.0) i->r = 1.0;
			if(i->r < 0.0) i->r = 0.0;
			if(i->g > 1.0) i->g = 1.0;
			if(i->g < 0.0) i->g = 0.0;
			if(i->b > 1.0) i->b = 1.0;
			if(i->b < 0.0) i->b = 0.0;
		}
	
		if(!f) throw QString("Unexpected end of file!");
	}
	catch(QString errormsg)
	{
		f.close();
		throw errormsg;
	}
	
	accessmutex.unlock();	
	
	// signalling new data to view?
	
	f.close();
	
  modified=false;
  m_filename=filename;
	m_title=QFileInfo(filename).fileName();
  		
  return true;
}

bool QtpLgviewerDoc::saveDocument(const QString &filename, const char *format /*=0*/)
{
	ofstream f( filename );
	if (!f) return false;

	// remember: point and color indices are converted to c array convertion (add 1 to each element...)
	// write header information
	f << pointlist.size() << " " << polylist.size() << " " << colortable.size() << endl;
	// write point list, convert from right-handed internal system to the left-handed external system
	f.setf(ios::left);
	for(vector<MyPoly3D::point4f>::iterator i = pointlist.begin(); i != pointlist.end(); i++)
		f << setw(10) << (i->x() / i->w()) << ' ' << setw(10) << (i->y() / i->w()) << ' ' << setw(10) << (i->z() / i->w()) << endl;
	// write poly list
	for(vector<MyPoly3D::poly>::iterator i = polylist.begin(); i != polylist.end(); i++)
		f << (i->p1 + 1) << '\t' << (i->p2 + 1) << '\t' << (i->p3 + 1) << '\t' << (i->cindex + 1) << endl;
	// write color table
	for(vector<MyPoly3D::color3f>::iterator i = colortable.begin(); i != colortable.end(); i++)
		f << i->r << ' ' << i->g << ' ' << i->b << endl;

  f.close();

  modified=false;
  m_filename=filename;
	m_title=QFileInfo(filename).fileName();
  return true;
}

void QtpLgviewerDoc::deleteContents()
{

	// lock access to data
	while(!accessmutex.lock())
	
	pointlist.clear();
	polylist.clear();
	colortable.clear();
	
	accessmutex.unlock();
}

bool QtpLgviewerDoc::canCloseFrame(QtpLgviewerView* pFrame)
{
	if(!isLastView())
		return true;
		
	bool ret=false;
  if(isModified())
  {
		QString saveName;
  	switch(QMessageBox::information(pFrame, title(), tr("The current file has been modified.\n"
                          "Do you want to save it?"),QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel ))
    {
			case QMessageBox::Yes:
				if(title().contains(tr("Untitled")))
				{
					saveName=QFileDialog::getSaveFileName(0, 0, pFrame);
                                          if(saveName.isEmpty())
          	                                return false;
				}
				else
					saveName=pathName();

				if(!saveDocument(saveName))
				{
 					switch(QMessageBox::critical(pFrame, tr("I/O Error !"), tr("Could not save the current document !\n"
																												"Close anyway ?"),QMessageBox::Yes ,QMessageBox::No))

 					{
 						case QMessageBox::Yes:
 							ret=true;
 						case QMessageBox::No:
 							ret=false;
 					}
				}
				else
					ret=true;
				break;
			case QMessageBox::No:
				ret=true;
				break;
			case QMessageBox::Cancel:
			default:
				ret=false;
				break;
		}
	}
	else
		ret=true;

	return ret;
}
