/***************************************************************************
                          qtplgviewerview.h  -  description
                             -------------------
    begin                : Don Jun 28 10:55:59 CEST 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QTPLGVIEWERVIEW_H
#define QTPLGVIEWERVIEW_H

// include files for Qt
#include <qwidget.h>
#include <qpixmap.h>
#include <qpointarray.h>
#include <qstring.h>

// other includes
#include "mypoly3d.h"
#include <vector>
#include "mymutex.h"

class QtpLgviewerDoc;
class QColor;

/**
 * This class provides an incomplete base for your application view.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 1.1 code generation
 */
class QtpLgviewerView : public QWidget
{
  Q_OBJECT

  friend QtpLgviewerDoc;

  public:
  	enum view_flags {none = 0, wireframe=1,backfaceculling=2,flatshading=4};
  	
    /** Constructor for the view
    	* @param pDoc  your document instance that the view represents. Create a document before calling the constructor
    	* or connect an already existing document to a new MDI child widget.*/
    QtpLgviewerView(QtpLgviewerDoc* pDoc, QWidget* parent, const char *name, int wflags);
    /** Destructor for the main view */
    ~QtpLgviewerView();
		/** returns a pointer to the document connected to the view*/
    QtpLgviewerDoc *getDocument() const;
    /** gets called to redraw the document contents if it has been modified */
		void update(QtpLgviewerView* pSender);
    /** contains the implementation for printing functionality and gets called by QtpLgviewerApp::slotFilePrint() */
    void print(QPrinter *pPrinter);
		/** getter for the current view type */
		view_flags getViewFlags() { return viewtype; }
		/** getter for the view transform */
		MyPoly3D::tmatrix::viewtransfrom_t getViewTransform() { return viewtransform; }
		/** getter for the focal length */
		float getFocalLength() { return bw; }
		/** getter for the background color */
		const QColor& getBackgroundColor() { return backgroundcolor; }
	signals: // Signals
  	/** signals view settings to application class */
  	void signalViewSettings();
  	
  protected:
	  virtual void closeEvent(QCloseEvent*);
		/** paints the widget content */
	  void paintEvent(QPaintEvent *);
	  /** resizes the backbuffer pixmap */
	  void resizeEvent(QResizeEvent *);
	  /** for keyboard camera & view scaling control */
	  void keyPressEvent(QKeyEvent *);
	  /** for updating the toolbars (view transform & mode) */
	  void focusInEvent(QFocusEvent *) { emit signalViewSettings(); }
		/** reference to the document */
	  QtpLgviewerDoc *doc;
	  /** color table */
	  vector<QColor> colortable;
	  /** object transformation matrix */
	  MyPoly3D::tmatrix objtransf;
	  /** view transformation matrix */
		MyPoly3D::tmatrix viewtransf;
		/** focal length of camera lens */
		float bw;
		/** scaling of view */
		float scale;
		/** backbuffer pixmap */
		QPixmap	backbuffer;
		/** background color */
		QColor backgroundcolor;
		/** view type */
		view_flags viewtype;
		/** view transform */
		MyPoly3D::tmatrix::viewtransfrom_t viewtransform;
		/** Mutex for synchronizing the access to the backbuffer */
		MyMutex backbuffermutex;
		/** contains the point coordinates of a polygon in the view plane */
		QPointArray projpolygon;
		/** contains the point coordinates of a polygon */
		MyPoly3D::point4f polygon[3];
		/** caption got from doc object */
		QString mycaption;
	
  private:
	
public slots: // Public slots
  /** repaints the backbuffer */
  void slotUpdateBackBuffer();
  /** rereads color table from document */
  void slotUpdateColorTable();
  /** setter for the background color */
  void slotSetBackgroundColor(QColor c) {backgroundcolor = c; setBackgroundColor(backgroundcolor); slotUpdateBackBuffer(); repaint(false);}
  /** setter for the view type */
  void slotSetViewType(QtpLgviewerView::view_flags type) {viewtype = type; slotUpdateBackBuffer(); repaint(false);}
  /** setter for the view transform type */
  void slotSetViewTransform(MyPoly3D::tmatrix::viewtransfrom_t viewtransform);
  /** slot for setting the focal length of the camera lens */
  void slotSetFocalLength(float bw) { if(bw > 0.0) this->bw = bw; }
  /** slot for scaling the view up */
  void slotScaleUp() { scale *= 2.0; slotUpdateBackBuffer(); repaint(false);}
  /** slot for scaling the view down */
  void slotScaleDown() { if(scale / 2.0 > 0.0) scale /= 2.0; slotUpdateBackBuffer(); repaint(false);}
  /** slot for setting the scale to 1:1 */
  void slotScaleOnetoOne() { scale = 1.0; slotUpdateBackBuffer(); repaint(false);}
  /** setting the scale to a concrete value */
  void slotSetScale(float scale) { if(scale > 0.0) this->scale = scale; slotUpdateBackBuffer(); repaint(false);}
  /** slot moving the _camera_ right */
  void slotStepRight() {objtransf.translate(-1.0/scale,0,0); slotUpdateBackBuffer(); repaint(false); }
	/** slot moving the _camera_ left */
  void slotStepLeft() {objtransf.translate(1.0/scale,0,0); slotUpdateBackBuffer(); repaint(false); }
	/** slot moving the _camera_ up */
  void slotStepUp() {objtransf.translate(0,-1.0/scale,0); slotUpdateBackBuffer(); repaint(false); }
  /** slot moving the _camera_ down */
  void slotStepDown() {objtransf.translate(0,1.0/scale,0); slotUpdateBackBuffer(); repaint(false); }
  /** slot moving the _camera_ up */
	void slotStepForward() {objtransf.translate(0,0,-1.0/scale); slotUpdateBackBuffer(); repaint(false); }
	/** slot moving the _camera_ up */
	void slotStepBackward() {objtransf.translate(0,0,1.0/scale); slotUpdateBackBuffer(); repaint(false); }
	/** slot rotating the _camera_ right */
	void slotLookRight() {objtransf.rotateAroundY(-10.0); slotUpdateBackBuffer(); repaint(false); }
	/** slot rotating the _camera_ left */
	void slotLookLeft() {objtransf.rotateAroundY(10.0); slotUpdateBackBuffer(); repaint(false); }
	/** slot rotating the _camera_ up */
	void slotLookUp() {objtransf.rotateAroundX(10.0); slotUpdateBackBuffer(); repaint(false); }
	/** slot rotating the _camera_ down */
	void slotLookDown() {objtransf.rotateAroundX(-10.0); slotUpdateBackBuffer(); repaint(false); }
  /** No descriptions */
  void slotUpdateName(const QString &basename = "");
  /** set base name of window */
  void slotSetBaseName(const QString &name) {mycaption = name; }
};


#endif
